// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// File:        db.js --> javascript library for DB functions
// Author:      Constantinos Valakas
// Copyright:   Constantinos Valakas
// Project:     Eortologio mozilla firefox extension
// Version:     1.2
// Framework:   2.0
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*
    Eortologio - Greek Namedays Calendar
    Copyright (C) 2010  - Constantinos Valakas

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

if(!com) var com = {};
if(!com.cvalakas) com.cvalakas = {};
if(!com.cvalakas.eortologio) com.cvalakas.eortologio = {};
if(!com.cvalakas.eortologio.Database) com.cvalakas.eortologio.Database = {};

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

com.cvalakas.eortologio.Database = function()
{
   // ******** PRIVATE MEMBERS

   // ******** PRIVATE METHODS

   // ******** PUBLIC METHODS AND PROPERTIES
   return {
      // ******** PUBLIC PROPERTIES

      // ******** PUBLIC METHODS

		/**
		* Opens an SQLlite database connection.
		* @author Constantinos Valakas
		* @return A SQLlite database connection.
		*/
		openDatabase: function()
		{
			try
			{
				var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
				file.initWithPath(chromeToPath("chrome://eortologio/locale/eortologio.sqlite"));
    
				var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);  
				var mDBConn = storageService.openDatabase(file); // Will also create the file if it does not exist
    
				return mDBConn;
			}
			catch(e)
			{
				alert("openDatabase(): " + e);
				Components.utils.reportError("openDatabase(): " + e);
			}
		},

		/**
		* Opens the SQLlite user specific database connection.
		* @author Constantinos Valakas
		* @return A SQLlite database connection.
		*/
		openUserDatabase: function(name)
		{
			try
			{
				var file = Components.classes["@mozilla.org/file/directory_service;1"]  
							.getService(Components.interfaces.nsIProperties)  
							.get("ProfD", Components.interfaces.nsIFile);
							file.append(name); 

				// Get storage service
				var storageService = Components.classes["@mozilla.org/storage/service;1"]  
									.getService(Components.interfaces.mozIStorageService);  
				// ... and open a connection to user's database
				var mDBConn = storageService.openDatabase(file);
				// Return connection
				return mDBConn;
			}
			catch(e)
			{		
				alert(e);
				Components.utils.reportError(e);
			}
		},

		/**
		* Opens the SQLlite user specific database connection.
		* @author Constantinos Valakas
		* @return A SQLlite database connection.
		*/
		checkUserDBFile: function(name)
		{
			try
			{
				// Get the file location
				var file = Components.classes["@mozilla.org/file/directory_service;1"]  
							.getService(Components.interfaces.nsIProperties)  
							.get("ProfD", Components.interfaces.nsIFile);
				file.append(name); 

				// Check and see if the file exists, if not the user uses this file for first time
				// Copy the default file from the extension's defaults directory
				if( !file.exists() )
				{
					// Copy the empty file from the extension's "defaults" directory
					var MY_ID = "{7cbb3fa0-b0ad-11dd-ad8b-0800200c9a66}";
					var em = Components.classes["@mozilla.org/extensions/manager;1"].
					getService(Components.interfaces.nsIExtensionManager);

					// Locate the "defaults" directory
					var srcFile = em.getInstallLocation(MY_ID).getItemFile(MY_ID, "defaults/" + name);
					if( srcFile.exists() )
					{
						// Copy to user's firefox profile directory (so in case of a new version install the user's
						// data will be reserved.
						srcFile.copyTo(Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("ProfD", Components.interfaces.nsIFile), null);
					}
					else alert("Extension's directory has been corrupted. Please reinstall extension. We're sorry for the inconvienience...");
				}
				
				// Set read and write permissions by default (mostly for linux versions)
				file.permissions = 384;
			}
			catch(e)
			{		
				alert("checkUserDBFile(): " + e);
				Components.utils.reportError("checkUserDBFile(): " + e);
			}
		}
	};   // End of public module
}();    // End of class
