4// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// File:        eortologio.js --> javascript functions for the Eortologio mozilla extension
// Author:      Constantinos Valakas
// Copyright:   Constantinos Valakas
// Project:     Eortologio mozilla firefox extension
// Version:     1.2.2
// Framework:   2.0
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*
    Eortologio - Greek Namedays Calendar
    Copyright (C) 2010  - Constantinos Valakas

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

if(!com) var com = {};
if(!com.cvalakas) com.cvalakas = {};
if(!com.cvalakas.eortologio) com.cvalakas.eortologio = {};
if(!com.cvalakas.eortologio.Eortologio) com.cvalakas.eortologio.Eortologio = {};

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

com.cvalakas.eortologio.Eortologio = function()
{
	// ******** PRIVATE MEMBERS
	var diffNameday = 0;
	var timer = null;

	// ******** PRIVATE METHODS
	/**
	* Returns a date string in d/M format.
    * @author Constantinos Valakas
    * @param A given date to be transformed.
    * @return A string in "d/M" format.
	*/
	var getEortologioDate = function (dateObj)
	{
		var day, month;

		day = dateObj.getDate();
		month = dateObj.getMonth() + 1;

		if( String(day).length < 2) day = '0' + day;
		if( String(month).length < 2 ) month = '0' + month;
   
		return day + "/" + month;  
	}

	/**
	* Sets the celebrating names and display them.
    * @author Constantinos Valakas
    */
	var setNamedays = function()
	{
		var popDate;
		var today = new Date();
		var specialQueryStr = "";

		try
		{
			// Grab the user preferences
			var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.eortologio.");

			// Check user preference if the personal note of tomorrow should be displayed today
			if( prefs.getBoolPref("noteOnPreviousDay") )
				popDate = getEortologioDate(new Date( today.getTime() + 86400000));
			else
				popDate = getEortologioDate(today);
      
			// ******** Calculate dates and set namedays ********
			diffNameday = com.cvalakas.eortologio.MoveableNamedays.calculateNamedayDifference( today.getFullYear() );
			specialQueryStr = com.cvalakas.eortologio.MoveableNamedays.getSpecialNamedayQuery();

			// Set data sources according to selected namedays language
			document.getElementById("tooltip-names-mov-query").textContent = "SELECT primary_name, rest_names FROM mov_names " +
																			 "WHERE days_to_eastern = '" + diffNameday + 
																			 "' ORDER BY priority";
			document.getElementById("tooltip-names-standard-query").textContent = "SELECT primary_names, rest_names FROM names WHERE date = '" + 
																				 getEortologioDate(today) +
																				 "' ORDER BY priority LIMIT 2;";
			document.getElementById("tooltip-names-special-query").textContent = specialQueryStr;
	
			document.getElementById("statusbar-names-mov-query").textContent = "SELECT primary_name, rest_names FROM mov_names " +
																			   "WHERE days_to_eastern = '" + diffNameday + 
																			   "' ORDER BY priority LIMIT 1";
			document.getElementById("statusbar-names-special-query").textContent = specialQueryStr + " LIMIT 1";
			document.getElementById("statusbar-names-standard-query").textContent = "SELECT primary_names, rest_names FROM names WHERE date = '" + 
																					getEortologioDate(today) +
																					"' ORDER BY priority LIMIT 2;";

			//document.getElementById("statusbar-names-standard-query-param").textContent = getEortologioDate(today);      
      
			document.getElementById("note-comments-vbox").setAttribute("datasources", "profile:eortologio-user-notes.sqlite");      
			document.getElementById("tooltip-query-note").textContent = "SELECT n_comment FROM notes " +
																		"WHERE n_date = '" + popDate + "';";

			// *** Rebuild components
			// Status-bar
			document.getElementById("statusbar-names-mov-hbox").builder.rebuild();
			document.getElementById("statusbar-names-special-hbox").builder.rebuild();
			document.getElementById("statusbar-names-standard-hbox").builder.rebuild();
			
			// Tooltip
			document.getElementById("tooltip-names-mov-vbox").builder.rebuild();
			document.getElementById("tooltip-names-standard-vbox").builder.rebuild();
			document.getElementById("tooltip-names-special-vbox").builder.rebuild(); 

			// Checking
			//var dt = new Date();
			//Components.utils.reportError("Timeout: " + dt.getHours() + ":" + dt.getMinutes() + ":" + dt.getSeconds());
		}
		catch(e)
		{
			alert("setNamedays(): " + e);
			Components.utils.reportError("setNamedays(): " + e);
		}
	}

	// ******** PUBLIC METHODS AND PROPERTIES
	return {	// <-- THIS BRACKET SHOULD BE LOCATED HERE! DON'T ASK WHY!
		// ******** PUBLIC PROPERTIES

		// ******** PUBLIC METHODS

		/**
		* Clears the search list box object.
		* @author Constantinos Valakas
		*/
		clearListbox: function ()
		{
			var i;
			var itemsNr = document.getElementById("search-results").itemCount;

			i=0; 
			while( document.getElementById("search-results").itemCount != 0 )
			{
				document.getElementById("search-results").removeItemAt(itemsNr-i);
				i++;
			}
		},

		/**
		* Searches a nameday into database.
		* @author Constantinos Valakas
		* @param type the type of search (by date or by name).
		* @param value the search criteria.
		*/
		searchNameday: function ( type, value )
		{
			var queryStr;
			var qValue = "", qMonths = "", qDays = "";

			// Grab the user preferences
			var prefs = Components.classes["@mozilla.org/preferences-service;1"]
							.getService(Components.interfaces.nsIPrefService)
							.getBranch("extensions.eortologio.");

			// Define datasource according to locale
			document.getElementById("search-results").setAttribute("datasources", "chrome://eortologio/content/data/" + prefs.getCharPref("namedaysLocale") + "/eortologio.sqlite");

			if( type == "BYDATE" )
			{  
				// Retrieve the selected date (format: YYYY-MM-DD)
				qValue = document.getElementById("search-bydate-datepicker").value;
				qMonths = qValue.substring(5,7);
				qDays = qValue.substring(8,10);
				qValue = qDays + "/" + qMonths;

				// Construct SQL query
				queryStr = "SELECT primary_names, rest_names FROM names WHERE date = '" + qValue + "'";
     
				document.getElementById("search-query").textContent = queryStr;
				document.getElementById("search-results-item1").label = "?primary_names" + "  " + "?rest_names";
				document.getElementById("search-results").builder.rebuild();
			}
			else if( type == "BYNAME" )
			{
				// Construct SQL query
				queryStr = "SELECT date, primary_names FROM names WHERE primary_names LIKE '%" + value + "%' OR rest_names LIKE '%" + value + "%'";
       
				document.getElementById("search-query").textContent = queryStr;
				document.getElementById("search-results-item1").label = "?date | ?primary_names";
				document.getElementById("search-results").builder.rebuild();
			}
		},

		/**
		* The delegate function of the load event.
		* @author Constantinos Valakas
		*/
		eLoad: function()
		{
			var today, popDate;
			var specialQueryStr = "";
			var i;
			var noteOnPreviousDay = false;
   
			try
			{
				// Check if the user personalized database files exist in the "profile" directory
				com.cvalakas.eortologio.Database.checkUserDBFile("eortologio-user-notes.sqlite");
	   
				// Grab the user preferences
				var prefs = Components.classes["@mozilla.org/preferences-service;1"]
								.getService(Components.interfaces.nsIPrefService)
								.getBranch("extensions.eortologio.");

				// Grab the browser preferences
				var mozPrefs = Components.classes["@mozilla.org/preferences-service;1"]
								.getService(Components.interfaces.nsIPrefService)
								.getDefaultBranch("general.useragent.");

				// Check for namedays display language
				if( prefs.getCharPref("namedaysLocale") == "" || prefs.getCharPref("namedaysLocale") == null )
				{
					// Set default namedays language
					//prefs.setCharPref("namedaysLocale", mozPrefs.getComplexValue("locale", Components.interfaces.nsIPrefLocalizedString).data);   // ATTENTION: we use this way ot get the specific preference
					prefs.setCharPref("namedaysLocale", "el-GR");
				}

				// **** Define datasources according to locale
				// Status-bar
				document.getElementById("statusbar-names-standard-hbox").setAttribute("datasources", "chrome://eortologio/content/data/" + prefs.getCharPref("namedaysLocale") + "/eortologio.sqlite");
				document.getElementById("statusbar-names-mov-hbox").setAttribute("datasources", "chrome://eortologio/content/data/" + prefs.getCharPref("namedaysLocale") + "/eortologio.sqlite");
				document.getElementById("statusbar-names-special-hbox").setAttribute("datasources", "chrome://eortologio/content/data/" + prefs.getCharPref("namedaysLocale") + "/eortologio.sqlite");
				// Tooltip
				document.getElementById("tooltip-names-special-vbox").setAttribute("datasources", "chrome://eortologio/content/data/" + prefs.getCharPref("namedaysLocale") + "/eortologio.sqlite");
				document.getElementById("tooltip-names-mov-vbox").setAttribute("datasources", "chrome://eortologio/content/data/" + prefs.getCharPref("namedaysLocale") + "/eortologio.sqlite");
				document.getElementById("tooltip-names-standard-vbox").setAttribute("datasources", "chrome://eortologio/content/data/" + prefs.getCharPref("namedaysLocale") + "/eortologio.sqlite");
				// Note panel
				document.getElementById("note-comments-vbox").setAttribute("datasources", "chrome://eortologio/content/data/" + prefs.getCharPref("namedaysLocale") + "/eortologio.sqlite");

				today = new Date();

				// ******** Set label colors according to user's preferences *********
				// Set standard namedays label
	  
				if( !prefs.getBoolPref("labelNoBackgroundColor") )
				{
					document.getElementById('eortologio-statusbar-label').style.backgroundColor = prefs.getCharPref("labelBackgroundColor");
					document.getElementById('eortologio-statusbar-label').style.color = prefs.getCharPref("labelForegroundColor");
				}
				else
				{
					document.getElementById('eortologio-statusbar-label').style.backgroundColor = "";
					document.getElementById('eortologio-statusbar-label').style.color = "#000000";
				}	  
				if( prefs.getBoolPref("labelBoldFonts") )
				{
					document.getElementById('eortologio-statusbar-label').style.fontWeight = "bold";
				}
				document.getElementById("eortologio-statusbar-label").style.padding = "2px";
				document.getElementById("statusbar-names-standard-hbox").builder.rebuild();		  
	 
				// Set moveable namedays label
				if( document.getElementById('eortologio-statusbar-label-mov') )
				{
					if( !prefs.getBoolPref("labelNoBackgroundColor") )
					{
						document.getElementById('eortologio-statusbar-label-mov').style.backgroundColor = prefs.getCharPref("labelBackgroundColor");
						document.getElementById('eortologio-statusbar-label-mov').style.color = prefs.getCharPref("labelForegroundColor");
					}
					else
					{
						document.getElementById('eortologio-statusbar-label-mov').style.backgroundColor = "";
						document.getElementById('eortologio-statusbar-label-mov').style.color = "#000000";
					}		  
					if( prefs.getBoolPref("labelBoldFonts") )
					{
						document.getElementById('eortologio-statusbar-label-mov').style.fontWeight = "bold";
					}
					document.getElementById('eortologio-statusbar-label-mov').style.padding = "2px";
				}
			
				// Set special moveable namedays label
				if( document.getElementById('eortologio-statusbar-label-special') )
				{
					if( !prefs.getBoolPref("labelNoBackgroundColor") )
					{
						document.getElementById('eortologio-statusbar-label-special').style.backgroundColor = prefs.getCharPref("labelBackgroundColor");  
						document.getElementById('eortologio-statusbar-label-special').style.color = prefs.getCharPref("labelForegroundColor");
					}		
					else
					{
					document.getElementById('eortologio-statusbar-label-special').style.backgroundColor = "";
					document.getElementById('eortologio-statusbar-label-special').style.color = "#000000";
					}		  
					if( prefs.getBoolPref("labelBoldFonts") )
					{
						document.getElementById('eortologio-statusbar-label-special').style.fontWeight = "bold";
					}
					document.getElementById('eortologio-statusbar-label-special').style.padding = "2px";
				}
	  
				// Check date and set namedays (tooltip, note, statusbar) and do it so every 1 minute
				setNamedays();
				// Use mozilla API nsITimer instead of setT1meout() or set1nterval() functions due to security reasons
				var event = { notify: function(timer) { setNamedays(); } }
				// Create timer instance
				timer = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
				timer.initWithCallback(event, 60000, Components.interfaces.nsITimer.TYPE_REPEATING_SLACK);
				//window.set1nterval( function(){setNamedays();}, 60*1000);		// <-- PAY ATTENTION HOW THE FUNCTION IS CALLED!

				// Check user preference if the personal note of tomorrow should be displayed today
				if( prefs.getBoolPref("noteOnPreviousDay") )
					popDate = getEortologioDate(new Date( today.getTime() + 86400000));
				else
					popDate = getEortologioDate(today);

				// ******** Personal note ********
				if (com.cvalakas.eortologio.Note.checkNote(popDate) )
				{  
					// Use mozilla API nsITimer instead of setT1meout() or set1nterval() functions due to security reasons
					var event1 = { notify: function(timer1) { document.getElementById("notetip").openPopup( document.getElementById("eortologio-statusbar-panel"), "before_start", 5, -5, false, false );} }
					var event2 = { notify: function(timer2) { document.getElementById("notetip").hidePopup(); } }
               
					// Create timer instance
					var timer1 = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
					var timer2 = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);

					// Give some time for all extensions in status bar to load and
					// finalize that way the position of this extension in status bar
					timer1.initWithCallback(event1, 2000, Components.interfaces.nsITimer.TYPE_ONE_SHOT);
                  
					// Hide personal note after 10 seconds
					timer2.initWithCallback(event2, 10000, Components.interfaces.nsITimer.TYPE_ONE_SHOT);               
				}

				// *** Rebuild components
				// Status-bar
				document.getElementById("statusbar-names-mov-hbox").builder.rebuild();
				document.getElementById("statusbar-names-special-hbox").builder.rebuild();
				document.getElementById("statusbar-names-standard-hbox").builder.rebuild();
				// Note panel
				document.getElementById("note-comments-vbox").builder.rebuild();
				// Tooltip
				document.getElementById("tooltip-names-mov-vbox").builder.rebuild();
				document.getElementById("tooltip-names-standard-vbox").builder.rebuild();
				document.getElementById("tooltip-names-special-vbox").builder.rebuild(); 
			}
			catch(e)
			{
				alert("eLoad(): " + e);
				Components.utils.reportError("eLoad(): " + e);
			}
		},

		/**
		* The delegate function of the unload event.
		* @author Constantinos Valakas
		*/
		eUnload: function()
		{
		}
	};   // End of public module
}();	// End of class

