// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// File:        movableNamedays.js
// Author:      Constantinos Valakas
// Copyright:   Constantinos Valakas
// Project:     Eortologio mozilla firefox extension
// Version:     1.2
// Framework:   2.0
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*
    Eortologio - Greek Namedays Calendar
    Copyright (C) 2010  - Constantinos Valakas

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

if(!com) var com = {};
if(!com.cvalakas) com.cvalakas = {};
if(!com.cvalakas.eortologio) com.cvalakas.eortologio = {};
if(!com.cvalakas.eortologio.MoveableNamedays) com.cvalakas.eortologio.MoveableNamedays = {};

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

com.cvalakas.eortologio.MoveableNamedays = function()
{
   // ******** PRIVATE MEMBERS
   var canturies, rem19, diff;
   var day, month;
   var strDay, strMonth;
   var factorA, factorB, factorC, factorD, factorE; 

   // ******** PRIVATE METHODS

   /**
    * Calculates the Eastern (Holy Sunday) day feast.
    * @author Constantinos Valakas
    * @param year The current year.
    */
   var calculateEasternDate = function( year )
   {
      var centuries, temp;
	
      // Find the first 2 digits of the year (centuries)
      centuries = parseInt(year / 100);
    
      // Calculate first full moon after the Spring Equinox
      rem19 = year % 19;
      factorA = ((225-11 * rem19) % 30) + 21;
    
      // Calculate the nearest Sunday after that full moon
      factorB = (factorA - 19) % 7;
      factorC = (40 - centuries) % 7;
      temp = year % 100;
      factorD = (temp + parseInt(temp / 4)) % 7;
      factorE = ((20 - factorB - factorC - factorD) % 7) + 1;
    
      day = factorA + factorE;
    
      // Shift of 10 days from the Gregorian calendar
      temp = 10;
    
      if( year > 1600 )
      {
         temp = temp + centuries - 16 - parseInt((centuries - 16) / 4);
         day = day + temp;

         if( day > 61 )
         {
            day = day - 61;
            if( day < 10 ) strDay = "0" + day;                
            month = 5;  // May
            strMonth = "05";            
         }
         else if( day > 31 )
         {
            day = day - 31;
            if( day < 10 ) strDay = "0" + day;
            month = 4;  // April
            strMonth = "04";
         }
         else
         {
            month = 3;  // March
            strMonth = "03";
         }

         return new Date(year, month-1, day);
      }
   }

  

   /**
    * Auxiliary function, returns date in string format (d/m/yyyy).
    * @author Constantinos Valakas
    * @param date A Date object.
    * @return A string representind the given date in format: (d)d/(m)m/yyyy.
    */
   var dateToString = function( date )
   {
      var day, month, year;
   
      day = date.getDate();
      month = date.getMonth() + 1;
      year = date.getFullYear();

      return day + "/" + month + "/" + year;   
   }

   /**
    * Gets the current date.
    * @author Constantinos Valakas
    * @return String The given date in format "(d)d/(m)m. 
    */
   var getTodayDate = function()
   {
      var currentTime = new Date();
   
      var month = currentTime.getMonth() + 1;
      var day = currentTime.getDate();
   
      return day + "/" + month;   
   }

   // ******** PUBLIC METHODS AND PROPERTIES
   return {
      // ******** PUBLIC PROPERTIES

      // ******** PUBLIC METHODS
   
      /**
       * Calculates moveable namedays, according to the Eastern Holiday.
       * @author Constantinos Valakas
       * @param year A given year.
       * @return Time difference in days.
       */
      calculateNamedayDifference: function( year )
      {
         var date, today;
    
         // Check year to be between 1583 and 4099
         if( year < 1583 && year > 4099 )
         {
            return;
         }
    
         date = calculateEasternDate( year );
         today = new Date();
         today_compare = new Date(today.getFullYear(), today.getMonth(), today.getDate());
		 
         //alert( "today: " + today + "\n" + "eastern: " + date );
             
         // Return the time difference in days and compare with today_compare variable
		 // which is the today date in midnight (00:00) hour
         return (Math.round((today_compare - date)/1000/60/60/24));	
      },

      /**
       * Constructs the special moveable nameday query.
       * @author Constantinos Valakas
       * @return The special moveable nameday query string.
       */
      getSpecialNamedayQuery: function()
      {
         var queryStr = "";
         var todayStr, today;
         var tempDateStr, tempDate;
         var easternDate;
   
         // Check in sequence for special dates:
         // * a) 13/02:  If current date is equal, then check if it is Sunday, if yes query with standard_date.
         //              If not, find the date of next Sunday after 13/02 and check that date with current, if yes query with condition.
         // * b) 23/04:  If current date is equal, then check if Eastern Sunday is before 23/04, if yes query with standard_date.
         //              If not, get the Eastern date and check if Eastern Sunday + 1 day is equal to current date. If yes query woth condition.
         // * c) 25/04:  Follow the same algorithm as above, by searching Eastern Sunday + 2 days.
         // * d) 11/12:  If current date is equal, then check if it is Sunday, if yes query with standard_date.
         //              If not, find the date of next Sunday after 11/12 and check that date with current, if yes query with condition.

         today = new Date((new Date()).getFullYear(), (new Date()).getMonth(), (new Date()).getDate());
         todayStr = getTodayDate();
         easternDate = calculateEasternDate( today.getFullYear() );
   
         // **** (a) ****
         // Check current date
         if( todayStr == "13/2" )
         {
            // Check if it is Sunday (Sunday = 0)
            if( today.getDay() == 0 )
            {
               // Contruct query with standard_date
               queryStr = "SELECT primary_name, rest_names FROM special_names WHERE standard_date = '13/02'";
               return queryStr;      
            }
         }
         else
         {
            // Set a date of 13/02 (current year) (month-1)
            tempDate = new Date(today.getFullYear(), '01', '13');
            
            switch( tempDate.getDay() )
            {         
               case 1:        // Monday 
                  tempDate = new Date(today.getFullYear(), '01', '19');
                  break;
               case 2:        // Tuesday
                  tempDate = new Date(today.getFullYear(), '01', '18');
                  break;
               case 3:        // Wednesday
                  tempDate = new Date(today.getFullYear(), '01', '17');
                  break;
               case 4:        // Thursday
                  tempDate = new Date(today.getFullYear(), '01', '16');
                  break;
               case 5:        // Friday
                  tempDate = new Date(today.getFullYear(), '01', '15');
                  break;
               case 6:        // Saturday
                  tempDate = new Date(today.getFullYear(), '01', '14');
                  break;
               default:
                  break;            
            }
      
            // Compare current date with temp date
            if( dateToString(today) == dateToString(tempDate) )
            {
               // Dates match, construct query with condition (in this case condition = 1)
               queryStr = "SELECT primary_name, rest_names FROM special_names WHERE condition = 1";
               return queryStr;  
            }
         }
   
         // **** (b) ****
         // Check current date   
         if( todayStr == "23/4" )
         {
            // Check if Eastern is before April 23rd
            if( easternDate < today )      
            {
               // Construct query with standard_date
               queryStr = "SELECT primary_name, rest_names FROM special_names WHERE standard_date = '23/04'";
               return queryStr;         
            }
         }
         else
         {
            // CALCULATE SPECIAL NAMEDAY ONLY IF EASTERN IS AFTER APRIL 23RD!!!
            if( easternDate > new Date(today.getFullYear(), "03", "23") )
	    {
               // Get the next of Eastern
               tempDate = new Date(easternDate.getFullYear(), easternDate.getMonth(), easternDate.getDate()+1);

               // Compare it with current date
               if(dateToString(tempDate) == dateToString(today) )
               {
                  // Dates match, construct query with condition (in this case condition = 2)
                  queryStr = "SELECT primary_name, rest_names FROM special_names WHERE condition = 2";
                  return queryStr;    
               }
	    }
         }
  
         // **** (c) ****
         // Check current date
         if( todayStr == "25/4" )
         {
            // Check if Eastern is before April 23rd
            if( easternDate < (new Date(today.getFullYear(), '03', '23')) )      
            {
               // Construct query with standard_date
               queryStr = "SELECT primary_name, rest_names FROM special_names WHERE standard_date = '25/04'";
               return queryStr;         
            }
         }
         else
         {
            // Check if Eastern is after April 23rd
            if( easternDate > (new Date(today.getFullYear(), '03', '23')) )      
            {
   	       // Get the next of Eastern
               tempDate.setDate(easternDate.getDate() + 2);
      
               // Compare it with current date
               if( dateToString(tempDate) == dateToString(today) )
               { 
                  // Dates match, construct query with condition (in this case condition = 3)
                  queryStr = "SELECT primary_name, rest_names FROM special_names WHERE condition = 3";
                  return queryStr;    
               }
            }
         }
   
         // **** (d) ****
         // Check current date
         if( todayStr == "11/12" )
         {
            // Check if it is Sunday (Sunday = 0)
            if( today.getDay() == 0 )
            {
               // Contruct query with standard_date
               queryStr = "SELECT primary_name, rest_names FROM special_names WHERE standard_date = '11/12'";
               return queryStr;      
            }
         }
         else
         {
            // Set a date of 11/12 (current year) (month-1)
            tempDate = new Date(today.getFullYear(), '11', '11');
            switch( tempDate.getDay() )
            {         
               case 1:        // Monday 
                  tempDate = new Date(today.getFullYear(), '11', '17');
                  break;
               case 2:        // Tuesday
                  tempDate = new Date(today.getFullYear(), '11', '16');
                  break;
               case 3:        // Wednesday
                  tempDate = new Date(today.getFullYear(), '11', '15');
                  break;
               case 4:        // Thursday
                  tempDate = new Date(today.getFullYear(), '11', '14');
                  break;S
               case 5:        // Friday
                  tempDate = new Date(today.getFullYear(), '11', '13');
                  break;
               case 6:        // Saturday
                  tempDate = new Date(today.getFullYear(), '11', '12');
                  break;
               default:
                  break;            
            }
      
            // Compare current date with temp date
            if( dateToString(today) == dateToString(tempDate) )
            {
               // Dates match, construct query with condition (in this case condition = 4)
               queryStr = "SELECT primary_name, rest_names FROM special_names WHERE condition = 4";
               return queryStr;  
            }
         }
      }
   };
}();
