// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// File:        note.js --> javascript functions for the Eortologio mozilla extension - personal note
// Author:      Constantinos Valakas
// Copyright:   Constantinos Valakas
// Project:     Eortologio mozilla firefox extension
// Version:     1.2
// Framework:   2.0
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*
    Eortologio - Greek Namedays Calendar
    Copyright (C) 2010  - Constantinos Valakas

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

if(!com) var com = {};
if(!com.cvalakas) com.cvalakas = {};
if(!com.cvalakas.eortologio) com.cvalakas.eortologio = {};
if(!com.cvalakas.eortologio.Note) com.cvalakas.eortologio.Note = {};

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

com.cvalakas.eortologio.Note = function()
{
	// ******** PRIVATE MEMBERS

	// ******** PRIVATE METHODS
   
	/**
	* Auxiliary function. Transforms a chrome path to an actual path.
    * @author Constantinos Valakas
    * @param aPath The given chrome path.
    */
	var chromeToPath = function(aPath)
	{
		if (!aPath || !(/^chrome:/.test(aPath)))
			return; //not a chrome url
		var rv;
       
		var ios = Components.classes['@mozilla.org/network/io-service;1'].getService(Components.interfaces["nsIIOService"]);
		var uri = ios.newURI(aPath, "UTF-8", null);
		var cr = Components.classes['@mozilla.org/chrome/chrome-registry;1'].getService(Components.interfaces["nsIChromeRegistry"]);
		rv = cr.convertChromeURL(uri).spec;

		if (/^file:/.test(rv))
			rv = this.urlToPath(rv);
		else
			rv = this.urlToPath("file://"+rv);
		return(rv);
	}

	/**
    * Auxiliary function. Transforms a URL path to an actual path.
    * @author Constantinos Valakas
    * @param aPath The given URL path.
    */
	var urlToPath = function(aPath)
	{
		if (!aPath || !/^file:/.test(aPath))
			return ;
		var rv;
		var ph = Components.classes["@mozilla.org/network/protocol;1?name=file"]
							.createInstance(Components.interfaces.nsIFileProtocolHandler);
		rv = ph.getFileFromURLSpec(aPath).path;
		return rv;
	}

	// ******** PUBLIC METHODS AND PROPERTIES
	return {
		// ******** PUBLIC PROPERTIES

		// ******** PUBLIC METHODS

		/**
		* Clears note content textbox object.
		* @author Constantinos Valakas 
		*/
		clearTextbox: function()
		{
			// Clear note textbox value
			document.getElementById('note-comment-textbox').value = "";
		},

		/**
		* Inserts a personal note into database.
		* @author Constantinos Valakas
		* @param date the date when the personal note should be appeared.
		* @param note the content of the personal note. * 
		*/
		insertNote: function( date, note )
		{
			var qDate = "", qMonths = "", qDays = "";
      
			try
			{
				// Retrieve the selected date (format: YYYY-MM-DD)
				qDate = document.getElementById('note-date-datepicker').value;
				qMonths = qDate.substring(5,7);
				qDays = qDate.substring(8,10);
				qDate = qDays + "/" + qMonths;
      
				var mDBConn = com.cvalakas.eortologio.Database.openUserDatabase("eortologio-user-notes.sqlite");  
     
				mDBConn.executeSimpleSQL("INSERT INTO notes (n_date, n_comment) VALUES ( '" + 
										qDate + "', '" + document.getElementById('note-comment-textbox').value.replace("'", "`") + "' );");
				mDBConn.close();
			}
			catch(e)
			{
				//alert(e);	   
				//Components.utils.reportError(e);
			}   
		},

		/**
		* Deletes a personal note from database. All notes with the same date will be erased.
		* @author Constantinos Valakas
		* @param The listbox object of the available personal notes
		*/
		deleteNotes: function(notesList)
		{
			var i, qDate;
	
			try
			{	      
				var mDBConn = com.cvalakas.eortologio.Database.openUserDatabase("eortologio-user-notes.sqlite");
			
				for( i=0; i<notesList.selectedCount; i++ )
				{
					qDate = notesList.selectedItems[i].getElementsByTagName("listcell")[0].getAttribute("label");
					mDBConn.executeSimpleSQL("DELETE FROM notes WHERE n_date = '" + qDate + "';");
				}
				mDBConn.close();
			}
			catch(e)
			{
				//alert(e);
				//Components.utils.reportError(e);
			}
		},

		/**
		* Checks if the given date there is a personal note. 
		* @author Constantinos Valakas
		* @param date A the date to be checked.
		* @return True/False.
		*/
		checkNote: function(date)
		{ 
			var flag = false;
	
			try
			{	      
				var mDBConn = com.cvalakas.eortologio.Database.openUserDatabase("eortologio-user-notes.sqlite");  	     
				var statement = mDBConn.createStatement("SELECT n_comment FROM notes WHERE n_date = '" + date + "'");  
	   
				// Check to see if there is at least one row returned
				if( statement.executeStep() )
				{
					// statement.reset();
					flag = true;
				}
				else
				{
					// statement.reset();
					flag = false;
				}
	     
				return flag;	   
			}
			catch(e)
			{
				//alert(e);
				//Components.utils.reportError(e);
			}
		},

		/**
		* Saves the given user personalized sqlite file in a specific disk location.
		* @author Constantinos Valakas
		* @param path The path to the store location.
		* @param sqliteName The name of the personalized sqlite file. 
		* @param l10nMsg Localized info message.
		* @return True/False.
		*/
		backupNote: function(path, sqliteName, l10nMsgFailure, l10nMsgSuccess)
		{
			var flag = false;

			try
			{	      
				// Get the source file location
				var srcFile = Components.classes["@mozilla.org/file/directory_service;1"]  
									.getService(Components.interfaces.nsIProperties)  
									.get("ProfD", Components.interfaces.nsIFile);
				srcFile.append(sqliteName); 

				// Copy the sqlite file to user's desired location
				if( !srcFile.exists() )
				{
					alert(l10nMsgFailure);
				}
				else
				{
					var destFile = Components.classes["@mozilla.org/file/local;1"]
											 .createInstance(Components.interfaces.nsILocalFile);
					destFile.initWithPath(path);
			         
					srcFile.copyTo(destFile, srcFile.leafName + "." + new Date().getTime());

					alert(l10nMsgSuccess);
				}
			}
			catch(e)
			{
				//alert(e);
				//Components.utils.reportError(e);
			}
		}
	};   // End of public module
}();	 // End of class
